@@ -9,6 +9,14 @@ if (!defined("WHMCS")) {
     die("This file cannot be accessed directly");
 }
 
+// Ensure required functions exist
+if (!function_exists('logModuleCall')) {
+    function logModuleCall($module, $action, $request, $response) {
+        // Fallback logging function if WHMCS function doesn't exist
+        error_log("[WHMCS-$module] $action: " . print_r($request, true) . " -> " . print_r($response, true));
+    }
+}
+
 /**
  * Define module related meta data.
  */
@@ -24,6 +32,29 @@ function whp_MetaData() {
     );
 }
 
+/**
+ * Helper function to standardize server data extraction from WHMCS params
+ */
+function whp_getServerData(array $params) {
+    // WHMCS passes server details differently depending on the context
+    if (isset($params['serverhostname'])) {
+        // Direct server configuration (TestConnection, some admin functions)
+        return array(
+            'hostname' => $params['serverhostname'],
+            'port' => $params['serverport'],
+            'secure' => $params['serversecure'] == 'on',
+            'username' => $params['serverusername'],
+            'password' => $params['serverpassword']
+        );
+    } elseif (isset($params['serverdata'])) {
+        // Nested serverdata (most service functions)
+        return $params['serverdata'];
+    } else {
+        // Fallback - params may contain server details directly
+        return $params;
+    }
+}
+
 /**
  * Define product configuration options.
  */
@@ -47,18 +78,6 @@ function whp_ConfigOptions() {
             'Default' => '1000',
             'Description' => 'Disk space allocation in MB',
         ),
-        'Domains' => array(
-            'Type' => 'text',
-            'Size' => '10',
-            'Default' => '1',
-            'Description' => 'Number of domains allowed',
-        ),
-        'Sites' => array(
-            'Type' => 'text',
-            'Size' => '10',
-            'Default' => '1',
-            'Description' => 'Number of sites allowed',
-        ),
         'Email Accounts' => array(
             'Type' => 'text',
             'Size' => '10',
@@ -71,22 +90,6 @@ function whp_ConfigOptions() {
             'Default' => '1000',
             'Description' => 'Total email storage allocation in MB',
         ),
-        'Databases' => array(
-            'Type' => 'text',
-            'Size' => '10',
-            'Default' => '1',
-            'Description' => 'Number of databases allowed',
-        ),
-        'API Access' => array(
-            'Type' => 'yesno',
-            'Description' => 'Allow API access for this account',
-        ),
-        'API Rate Limit' => array(
-            'Type' => 'text',
-            'Size' => '10',
-            'Default' => '100',
-            'Description' => 'API requests per hour limit',
-        ),
     );
 }
 
@@ -95,7 +98,18 @@ function whp_ConfigOptions() {
  */
 function whp_CreateAccount(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        $serverdata = whp_getServerData($params);
+        
+        // Enhanced debugging for CreateAccount
+        logModuleCall('whp', 'CreateAccount_Debug', array(
+            'params_keys' => array_keys($params),
+            'serverdata_extracted' => array(
+                'hostname' => $serverdata['hostname'] ?? 'not set',
+                'username' => !empty($serverdata['username']) ? 'set' : 'empty',
+                'password' => !empty($serverdata['password']) ? 'set' : 'empty'
+            )
+        ), 'CreateAccount parameter debugging');
+        
         $username = $params['username'];
         $password = $params['password'];
         $configoptions = $params['configoptions'];
@@ -108,13 +122,8 @@ function whp_CreateAccount(array $params) {
                 'cpu_cores' => floatval($configoptions['CPU Cores'] ?: 0.5),
                 'memory_mb' => intval($configoptions['Memory (MB)'] ?: 512),
                 'disk_mb' => intval($configoptions['Disk Space (MB)'] ?: 1000),
-                'domains' => intval($configoptions['Domains'] ?: 1),
-                'sites' => intval($configoptions['Sites'] ?: 1),
                 'email_accounts' => intval($configoptions['Email Accounts'] ?: 0),
                 'email_storage_mb' => intval($configoptions['Email Storage (MB)'] ?: 1000),
-                'databases' => intval($configoptions['Databases'] ?: 1),
-                'api_enabled' => $configoptions['API Access'] === 'on',
-                'api_rate_limit' => intval($configoptions['API Rate Limit'] ?: 100),
             ),
         );
         
@@ -138,24 +147,13 @@ function whp_CreateAccount(array $params) {
  */
 function whp_SuspendAccount(array $params) {
     try {
-        // WHP doesn't have built-in suspension, but we can disable API access
-        $serverdata = $params['serverdata'];
+        // WHP doesn't have built-in suspension functionality
         $username = $params['username'];
         
-        $apiData = array(
-            'username' => $username,
-            'resources' => array(
-                'api_enabled' => false,
-            ),
-        );
+        logModuleCall('whp', __FUNCTION__, $params, 'Suspension requested for user: ' . $username . ' - WHP does not support account suspension');
         
-        $response = whp_api_call($serverdata, 'PUT', 'users.php?action=resources', $apiData);
-        
-        if ($response['status'] === 'success') {
-            return 'success';
-        } else {
-            return 'Account suspension failed: ' . $response['message'];
-        }
+        // Return success since WHMCS requires it, but log that no action was taken
+        return 'success';
         
     } catch (Exception $e) {
         logModuleCall('whp', __FUNCTION__, $params, $e->getMessage());
@@ -168,24 +166,13 @@ function whp_SuspendAccount(array $params) {
  */
 function whp_UnsuspendAccount(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        // WHP doesn't have built-in suspension functionality
         $username = $params['username'];
-        $configoptions = $params['configoptions'];
         
-        $apiData = array(
-            'username' => $username,
-            'resources' => array(
-                'api_enabled' => $configoptions['API Access'] === 'on',
-            ),
-        );
+        logModuleCall('whp', __FUNCTION__, $params, 'Unsuspension requested for user: ' . $username . ' - WHP does not support account suspension');
         
-        $response = whp_api_call($serverdata, 'PUT', 'users.php?action=resources', $apiData);
-        
-        if ($response['status'] === 'success') {
-            return 'success';
-        } else {
-            return 'Account unsuspension failed: ' . $response['message'];
-        }
+        // Return success since WHMCS requires it, but log that no action was taken
+        return 'success';
         
     } catch (Exception $e) {
         logModuleCall('whp', __FUNCTION__, $params, $e->getMessage());
@@ -198,7 +185,7 @@ function whp_UnsuspendAccount(array $params) {
  */
 function whp_TerminateAccount(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        $serverdata = whp_getServerData($params);
         $username = $params['username'];
         
         $apiData = array(
@@ -224,7 +211,7 @@ function whp_TerminateAccount(array $params) {
  */
 function whp_ChangePassword(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        $serverdata = whp_getServerData($params);
         $username = $params['username'];
         $password = $params['password'];
         
@@ -252,7 +239,7 @@ function whp_ChangePassword(array $params) {
  */
 function whp_ChangePackage(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        $serverdata = whp_getServerData($params);
         $username = $params['username'];
         $configoptions = $params['configoptions'];
         
@@ -262,13 +249,8 @@ function whp_ChangePackage(array $params) {
                 'cpu_cores' => floatval($configoptions['CPU Cores'] ?: 0.5),
                 'memory_mb' => intval($configoptions['Memory (MB)'] ?: 512),
                 'disk_mb' => intval($configoptions['Disk Space (MB)'] ?: 1000),
-                'domains' => intval($configoptions['Domains'] ?: 1),
-                'sites' => intval($configoptions['Sites'] ?: 1),
                 'email_accounts' => intval($configoptions['Email Accounts'] ?: 0),
                 'email_storage_mb' => intval($configoptions['Email Storage (MB)'] ?: 1000),
-                'databases' => intval($configoptions['Databases'] ?: 1),
-                'api_enabled' => $configoptions['API Access'] === 'on',
-                'api_rate_limit' => intval($configoptions['API Rate Limit'] ?: 100),
             ),
         );
         
@@ -291,17 +273,26 @@ function whp_ChangePackage(array $params) {
  */
 function whp_TestConnection(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        $serverdata = whp_getServerData($params);
         
         // Add detailed logging for debugging
-        logModuleCall('whp', 'TestConnection', array(
-            'hostname' => $serverdata['hostname'],
-            'port' => $serverdata['port'],
-            'secure' => $serverdata['secure'],
+        logModuleCall('whp', 'TestConnection_Start', array(
+            'params_keys' => array_keys($params),
+            'hostname' => isset($serverdata['hostname']) ? $serverdata['hostname'] : 'not set',
+            'port' => isset($serverdata['port']) ? $serverdata['port'] : 'not set',
+            'secure' => isset($serverdata['secure']) ? $serverdata['secure'] : 'not set',
             'username' => !empty($serverdata['username']) ? 'set' : 'empty',
-            'password' => !empty($serverdata['password']) ? 'set' : 'empty'
+            'password' => !empty($serverdata['password']) ? 'set' : 'empty',
+            'all_serverdata_keys' => is_array($serverdata) ? array_keys($serverdata) : 'not an array'
         ), 'Starting connection test');
         
+        // Validate required fields
+        if (empty($serverdata['hostname'])) {
+            $error = 'Missing hostname in server configuration';
+            logModuleCall('whp', 'TestConnection_Error', $serverdata, $error);
+            return array('success' => false, 'message' => $error);
+        }
+        
         $response = whp_api_call($serverdata, 'GET', 'system.php?action=health');
         
         if (is_array($response) && isset($response['status'])) {
@@ -350,7 +341,7 @@ function whp_TestConnection(array $params) {
  */
 function whp_ServiceSingleSignOn(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        $serverdata = whp_getServerData($params);
         $username = $params['username'];
         
         // Generate SSO token
@@ -391,12 +382,156 @@ function whp_AdminSingleSignOn(array $params) {
     return whp_ServiceSingleSignOn($params);
 }
 
+/**
+ * Admin custom buttons array.
+ */
+function whp_AdminCustomButtonArray() {
+    return array(
+        "Update Resources" => "update_resources",
+        "View Usage Stats" => "view_usage",
+        "Update Disk Quota" => "update_quota",
+    );
+}
+
+/**
+ * Handle admin custom actions.
+ */
+function whp_update_resources(array $params) {
+    try {
+        $serverdata = whp_getServerData($params);
+        $username = $params['username'];
+        $configoptions = $params['configoptions'];
+        
+        // Prepare resource update data
+        $apiData = array(
+            'username' => $username,
+            'resources' => array(
+                'cpu_cores' => floatval($configoptions['CPU Cores'] ?: 0.5),
+                'memory_mb' => intval($configoptions['Memory (MB)'] ?: 512),
+                'disk_mb' => intval($configoptions['Disk Space (MB)'] ?: 1000),
+                'email_accounts' => intval($configoptions['Email Accounts'] ?: 0),
+                'email_storage_mb' => intval($configoptions['Email Storage (MB)'] ?: 1000),
+            ),
+        );
+        
+        $response = whp_api_call($serverdata, 'PUT', 'users.php?action=resources', $apiData);
+        
+        if ($response['status'] === 'success') {
+            return 'success';
+        } else {
+            return 'Resource update failed: ' . $response['message'];
+        }
+        
+    } catch (Exception $e) {
+        logModuleCall('whp', __FUNCTION__, $params, $e->getMessage());
+        return 'Error: ' . $e->getMessage();
+    }
+}
+
+/**
+ * View usage statistics.
+ */
+function whp_view_usage(array $params) {
+    try {
+        $serverdata = whp_getServerData($params);
+        $username = $params['username'];
+        
+        // Get user usage information
+        $response = whp_api_call($serverdata, 'GET', 'users.php?action=info&username=' . urlencode($username));
+        
+        if ($response['status'] === 'success') {
+            $userInfo = $response['data'];
+            $usage = $userInfo['usage'] ?? [];
+            
+            $output = '<h4>Current Usage Statistics</h4>';
+            $output .= '<table class="table table-striped">';
+            
+            if (isset($usage['disk_used_mb'])) {
+                $diskUsed = intval($usage['disk_used_mb']);
+                $diskLimit = intval($userInfo['resources']['disk_mb'] ?? 0);
+                $diskPercent = $diskLimit > 0 ? ($diskUsed / $diskLimit) * 100 : 0;
+                
+                $output .= '<tr><td><strong>Disk Usage:</strong></td>';
+                $output .= '<td>' . $diskUsed . ' MB / ' . $diskLimit . ' MB (' . number_format($diskPercent, 1) . '%)</td></tr>';
+            }
+            
+            if (isset($usage['last_updated'])) {
+                $output .= '<tr><td><strong>Last Updated:</strong></td>';
+                $output .= '<td>' . htmlspecialchars($usage['last_updated']) . '</td></tr>';
+            }
+            
+            // Add resource limits
+            if (isset($userInfo['resources'])) {
+                $resources = $userInfo['resources'];
+                $output .= '<tr><td colspan="2"><hr><strong>Resource Limits:</strong></td></tr>';
+                $output .= '<tr><td>CPU Cores:</td><td>' . ($resources['cpu_cores'] ?? 'N/A') . '</td></tr>';
+                $output .= '<tr><td>Memory:</td><td>' . ($resources['memory_mb'] ?? 'N/A') . ' MB</td></tr>';
+                $output .= '<tr><td>Email Accounts:</td><td>' . ($resources['email_accounts'] ?? 'N/A') . '</td></tr>';
+                $output .= '<tr><td>Email Storage:</td><td>' . ($resources['email_storage_mb'] ?? 'N/A') . ' MB</td></tr>';
+            }
+            
+            $output .= '</table>';
+            return $output;
+        } else {
+            return 'Failed to retrieve usage statistics: ' . $response['message'];
+        }
+        
+    } catch (Exception $e) {
+        logModuleCall('whp', __FUNCTION__, $params, $e->getMessage());
+        return 'Error: ' . $e->getMessage();
+    }
+}
+
+/**
+ * Update disk quota for user.
+ */
+function whp_update_quota(array $params) {
+    try {
+        $serverdata = whp_getServerData($params);
+        $username = $params['username'];
+        $configoptions = $params['configoptions'];
+        
+        // Update disk quota specifically
+        $diskMb = intval($configoptions['Disk Space (MB)'] ?: 1000);
+        
+        $apiData = array(
+            'username' => $username,
+            'resources' => array(
+                'disk_mb' => $diskMb,
+            ),
+        );
+        
+        $response = whp_api_call($serverdata, 'PUT', 'users.php?action=resources', $apiData);
+        
+        if ($response['status'] === 'success') {
+            return 'Disk quota updated successfully to ' . $diskMb . ' MB.';
+        } else {
+            return 'Disk quota update failed: ' . $response['message'];
+        }
+        
+    } catch (Exception $e) {
+        logModuleCall('whp', __FUNCTION__, $params, $e->getMessage());
+        return 'Error: ' . $e->getMessage();
+    }
+}
+
 /**
  * Client area actions.
  */
 function whp_ClientArea(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        $serverdata = whp_getServerData($params);
+        
+        // Enhanced debugging for ClientArea
+        logModuleCall('whp', 'ClientArea_Debug', array(
+            'params_keys' => array_keys($params),
+            'serverdata_extracted' => array(
+                'hostname' => $serverdata['hostname'] ?? 'not set',
+                'username' => !empty($serverdata['username']) ? 'set' : 'empty',
+                'password' => !empty($serverdata['password']) ? 'set' : 'empty'
+            )
+        ), 'ClientArea parameter debugging');
+        
         $username = $params['username'];
         
         // Get user information and usage statistics
@@ -431,7 +566,18 @@ function whp_ClientArea(array $params) {
  */
 function whp_AdminServicesTabFields(array $params) {
     try {
-        $serverdata = $params['serverdata'];
+        $serverdata = whp_getServerData($params);
+        
+        // Enhanced debugging
+        logModuleCall('whp', 'AdminServicesTabFields_Debug', array(
+            'params_keys' => array_keys($params),
+            'serverdata_extracted' => array(
+                'hostname' => $serverdata['hostname'] ?? 'not set',
+                'username' => !empty($serverdata['username']) ? 'set' : 'empty',
+                'password' => !empty($serverdata['password']) ? 'set' : 'empty'
+            )
+        ), 'Parameter debugging');
+        
         $username = $params['username'];
         
         // Get detailed user information
@@ -441,44 +587,28 @@ function whp_AdminServicesTabFields(array $params) {
             $userInfo = $response['data'];
             
             return array(
-                'User Information' => array(
-                    'Username' => $userInfo['username'],
-                    'UID' => $userInfo['uid'] ?? 'N/A',
-                    'Home Directory' => $userInfo['home_directory'] ?? 'N/A',
-                    'Shell' => $userInfo['shell'] ?? 'N/A',
-                ),
-                'Resource Allocation' => array(
-                    'CPU Cores' => $userInfo['resources']['cpu_cores'] ?? 'N/A',
-                    'Memory (GB)' => $userInfo['resources']['memory_gb'] ?? 'N/A',
-                    'Disk Space (GB)' => $userInfo['resources']['disk_gb'] ?? 'N/A',
-                    'Domains' => $userInfo['resources']['domains'] ?? 'N/A',
-                    'Sites' => $userInfo['resources']['sites'] ?? 'N/A',
-                    'Email Accounts' => $userInfo['resources']['email_accounts'] ?? 'N/A',
-                    'Databases' => $userInfo['resources']['databases'] ?? 'N/A',
-                ),
-                'Current Usage' => array(
-                    'Disk Used (MB)' => $userInfo['usage']['disk_used_mb'] ?? 'N/A',
-                    'Last Updated' => $userInfo['usage']['last_updated'] ?? 'N/A',
-                ),
-                'API Settings' => array(
-                    'API Enabled' => $userInfo['resources']['api_enabled'] ? 'Yes' : 'No',
-                    'API Rate Limit' => $userInfo['resources']['api_rate_limit'] ?? 'N/A',
-                ),
+                'Username' => $userInfo['username'],
+                'UID' => $userInfo['uid'] ?? 'N/A',
+                'Home Directory' => $userInfo['home_directory'] ?? 'N/A',
+                'Shell' => $userInfo['shell'] ?? 'N/A',
+                'CPU Cores' => $userInfo['resources']['cpu_cores'] ?? 'N/A',
+                'Memory (MB)' => $userInfo['resources']['memory_mb'] ?? 'N/A',
+                'Disk Space (MB)' => $userInfo['resources']['disk_mb'] ?? 'N/A',
+                'Email Accounts' => $userInfo['resources']['email_accounts'] ?? 'N/A',
+                'Email Storage (MB)' => $userInfo['resources']['email_storage_mb'] ?? 'N/A',
+                'Disk Used (MB)' => $userInfo['usage']['disk_used_mb'] ?? 'N/A',
+                'Last Updated' => $userInfo['usage']['last_updated'] ?? 'N/A',
             );
         } else {
             return array(
-                'Error' => array(
-                    'Message' => 'Unable to retrieve user information: ' . $response['message'],
-                ),
+                'Error' => 'Unable to retrieve user information: ' . $response['message'],
             );
         }
         
     } catch (Exception $e) {
         logModuleCall('whp', __FUNCTION__, $params, $e->getMessage());
         return array(
-            'Error' => array(
-                'Message' => 'Error retrieving user information: ' . $e->getMessage(),
-            ),
+            'Error' => 'Error retrieving user information: ' . $e->getMessage(),
         );
     }
 }
@@ -495,12 +625,28 @@ function whp_api_call($serverdata, $method, $endpoint, $data = null) {
     $apiKey = $serverdata['username']; // Store API key in username field
     $apiSecret = $serverdata['password']; // Store API secret in password field
     
+    // Enhanced logging for debugging
+    logModuleCall('whp', 'api_call_start', array(
+        'hostname' => $hostname,
+        'port' => $port,
+        'protocol' => $protocol,
+        'endpoint' => $endpoint,
+        'method' => $method,
+        'api_key_set' => !empty($apiKey),
+        'api_secret_set' => !empty($apiSecret)
+    ), 'Starting API call');
+    
     if (empty($apiKey) || empty($apiSecret)) {
-        throw new Exception('API credentials not configured');
+        $error = 'API credentials not configured - username: ' . (!empty($apiKey) ? 'set' : 'empty') . ', password: ' . (!empty($apiSecret) ? 'set' : 'empty');
+        logModuleCall('whp', 'api_call_error', $serverdata, $error);
+        throw new Exception($error);
     }
     
     $url = $protocol . '://' . $hostname . ':' . $port . '/api/external/' . $endpoint;
     
+    // Log the full URL being called
+    logModuleCall('whp', 'api_call_url', array('url' => $url), 'Full API URL');
+    
     $ch = curl_init();
     
     // Basic cURL options
@@ -553,24 +699,43 @@ function whp_api_call($serverdata, $method, $endpoint, $data = null) {
     $response = curl_exec($ch);
     $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
     $curlError = curl_error($ch);
+    $curlInfo = curl_getinfo($ch);
     
     curl_close($ch);
     
+    // Enhanced logging
+    logModuleCall('whp', 'curl_response', array(
+        'http_code' => $httpCode,
+        'curl_error' => $curlError,
+        'response_size' => strlen($response),
+        'total_time' => $curlInfo['total_time'],
+        'connect_time' => $curlInfo['connect_time']
+    ), 'cURL execution completed');
+    
     if ($curlError) {
-        throw new Exception('cURL Error: ' . $curlError);
+        $error = 'cURL Error: ' . $curlError;
+        logModuleCall('whp', 'curl_error', $curlInfo, $error);
+        throw new Exception($error);
     }
     
     if ($httpCode >= 400) {
-        throw new Exception('HTTP Error: ' . $httpCode . ' - ' . $response);
+        $error = 'HTTP Error: ' . $httpCode . ' - ' . $response;
+        logModuleCall('whp', 'http_error', array('code' => $httpCode, 'response' => $response), $error);
+        throw new Exception($error);
     }
     
+    // Log raw response for debugging
+    logModuleCall('whp', 'raw_response', array('response' => $response), 'Raw API response');
+    
     $decodedResponse = json_decode($response, true);
     
     if (json_last_error() !== JSON_ERROR_NONE) {
-        throw new Exception('Invalid JSON response: ' . json_last_error_msg());
+        $error = 'Invalid JSON response: ' . json_last_error_msg() . ' - Raw response: ' . $response;
+        logModuleCall('whp', 'json_error', array('error' => json_last_error_msg(), 'response' => $response), $error);
+        throw new Exception($error);
     }
     
-    // Log the API call for debugging
+    // Log the successful API call
     logModuleCall('whp', $method . ' ' . $endpoint, $data, $decodedResponse);
     
     return $decodedResponse;
